h1, h2, h3, h4, h5, h6, p {line-height:1.2em; color:#E76300 }
h1 { font-size: 122%; font-weight:bold }
h2 { font-size: 107%; margin-top:1.5em; margin-bottom:.2em; padding:1em 0 0 0; border-top:1px dashed #C3D2DC; font-weight:bold}
h2.first { border-top:none; margin-top:0; margin-bottom:.2em}
p {color:#000; margin-bottom:1em }
#bd { padding-top: 1em }
#hd h1 { color:#ffffff }
#hd { width:750px; text-indent:15px; padding:4px 0 2px 0; background: url(background.png) 0 0 repeat-x #E76300; border: 1px solid #A55000; position:relative;}
#ft { font-size: 77%; padding-top:1em }
td { padding-right:0.2em }
ul {
  margin: 0;
  padding-left: 0em;
  list-style: none;
}

.orange_button {
  border:1px solid;
  border-color:#FC9 #630 #330 #F96;
  padding:0 3px;
  font:bold 10px verdana,sans-serif;
  color:#FFF;
  background:#F60;
  text-decoration:none;
  margin:0;
}

.PASS {
  color: #009900;
}

.FAIL {
  color: #990000;
}

.UNKNOWN {
  color: #999900;
  }

.NA {
  color: #999900;
}
